#!/usr/bin/env python
# encoding: utf-8
"""
Git IO utilities.
"""

import git


def absolute_git_root_dir(path):
    """Get the absolute path of the git repository root."""
    try:
        repo = git.Repo(path, search_parent_directories=True)
        return repo.git.rev_parse("--show-toplevel")
    except git.InvalidGitRepositoryError:
        return None


def read_git_blob(commit_ref, path, repo_dir='.'):
    """Read a file from a git blob."""
    repo = git.Repo(repo_dir)
    commit = repo.commit(commit_ref)
    blob = commit.tree / path
    return blob.data_stream.read().decode('utf-8')



#!/usr/bin/env python
# encoding: utf-8
"""
Command for running latexdiff.
"""

import logging
import os
import subprocess
import codecs
import shutil
import git

from .texutils import inline, inline_blob, remove_comments
from .gitio import read_git_blob, absolute_git_root_dir

from cliff.command import Command


class Diff(Command):
    """Run latexdiff between HEAD and a git ref."""

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(Diff, self).get_parser(prog_name)
        parser.add_argument(
            'prev_commit',
            help="Commit SHA to compare HEAD against.")
        parser.add_argument(
            '-n', '--name',
            default=None,
            help="Name of the difference file.")
        return parser

    def take_action(self, parsed_args):
        # Inline current and previous versions of the document
        if parsed_args.name is None:
            name = "current_{0}".format(parsed_args.prev_commit)
        else:
            name = parsed_args.name

        git_diff_pipeline(
            name,
            self.app.options.master,
            parsed_args.prev_commit)


def git_diff_pipeline(output_name, master_path, prev_commit):
    """Pipeline for typesetting latexdiff against a commit in git history."""
    log = logging.getLogger(__name__)

    current_path = inline_current(master_path)
    log.debug("current_path {0}".format(current_path))
    prev_path = inline_prev(prev_commit, master_path)
    log.debug("prev_path {0}".format(prev_path))

    # Run latexmk
    diff_path = os.path.splitext(output_name)[0]
    ldiff_cmd = "latexdiff --type=CTRADITIONAL {prev} {current} > {diff}.tex".\
        format(prev=prev_path, current=current_path, diff=diff_path)
    subprocess.call(ldiff_cmd, shell=True)

    # Compile the diff document with latexmk
    ltmk_cmd = "latexmk -f -pdf -bibtex-cond {0}.tex".format(diff_path)
    subprocess.call(ltmk_cmd, shell=True)

    # Copy to build directory
    if not os.path.exists("build"):
        os.makedirs("build")
    pdf_path = "{0}.pdf".format(output_name)
    if os.path.exists(pdf_path):
        shutil.move(pdf_path, os.path.join("build", pdf_path))

    # Clean up
    ltmk_cmd = "latexmk -f -pdf -bibtex-cond -c {0}.tex".format(diff_path)
    subprocess.call(ltmk_cmd, shell=True)
    build_exts = ['Notes.bib', '.bbl', '.tex']
    for ext in build_exts:
        path = "".join((output_name, ext))
        if os.path.exists(path):
            os.remove(path)
    os.remove(prev_path)
    os.remove(current_path)


def inline_current(root_tex_path):
    """Inline the current manuscript."""
    base_dir = os.path.dirname(root_tex_path)
    with codecs.open(root_tex_path, 'r', encoding='utf-8') as f:
        root_text = f.read()
        root_text = remove_comments(root_text)
        root_text = inline(root_text, base_dir=base_dir)
    output_path = "_current.tex"
    if os.path.exists(output_path):
        os.remove(output_path)
    with codecs.open(output_path, 'w', encoding='utf-8') as f:
        f.write(root_text)
    return output_path


def inline_prev(commit_ref, root_tex_path):
    """Inline the previous manuscript in the git tree.

    Parameters
    ----------
    commif_ref : str
        Commit reference string.
    root_tex_path : str
        Path to the root tex document in the filesystem.

    Returns
    -------
    output_path : str
        Path to the inlined latex document for latexdiff processing.
    """
    log = logging.getLogger(__name__)
    log.debug("inline_prev root_tex")
    log.debug(root_tex_path)
    git_root = absolute_git_root_dir(root_tex_path)
    rel_root_tex_path = os.path.relpath(os.path.abspath(root_tex_path),
                                        git_root)
    root_text = read_git_blob(commit_ref, rel_root_tex_path,
                              repo_dir=git_root)
    log.debug("prev root_text")
    log.debug(root_text)
    root_text = remove_comments(root_text)
    root_text = inline_blob(commit_ref, root_text,
                            base_dir=os.path.dirname(rel_root_tex_path),
                            repo_dir=git_root)
    output_path = "_prev.tex"
    if os.path.exists(output_path):
        os.remove(output_path)
    with codecs.open(output_path, 'w', encoding='utf-8') as f:
        f.write(root_text)
    return output_path


def get_n_commits():
    """Count commits in a repo from HEAD."""
    repo = git.Repo(".")
    commits = list(repo.iter_commits())
    n = len(commits)
    return n


def get_commits():
    """Returns a list of commits in the repository."""
    repo = git.Repo(".")
    commits = list(repo.iter_commits())
    return commits


def match_commit(sha):
    """Match the sha fragment to a commit."""
    commits = get_commits()
    for cm in commits:
        if cm.hexsha.startswith(sha):
            return cm
    return None



#!/usr/bin/env python
# encoding: utf-8
"""
Command for packaging the manuscript for submission.
"""

import logging
import os
import shutil
import codecs
import re
import subprocess

from .texutils import inline, remove_comments, inline_bbl, _find_exts

from cliff.command import Command


class Package(Command):
    """Package manuscript for arxiv/journal submission"""

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(Package, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            help="Name of packaged manuscript (saved to build/name).")
        parser.add_argument(
            '--style',
            default="aastex",
            choices=['aastex', 'arxiv'],
            help="Build style (aastex, arxiv).")
        parser.add_argument(
            '--exts',
            nargs='*',
            default=self.app.confs.config('exts') + ['png'],
            help="Figure extensions to use in order of priority")
        parser.add_argument(
            '--jpeg',
            action='store_true',
            default=False,
            help="Make JPEG versions of figures if too large (for arxiv)")
        parser.add_argument(
            '--maxsize',
            default=2.,
            type=float,
            help="Max figure size (MB) before converting to JPEG (for arxiv)")
        return parser

    def take_action(self, parsed_args):
        dirname = os.path.join("build", parsed_args.name)
        if not os.path.exists(dirname):
            os.makedirs(dirname)

        self._build_style = parsed_args.style
        self._ext_priority = parsed_args.exts
        self._max_size = parsed_args.maxsize

        bbl_path = ".".join((os.path.splitext(self.app.options.master)[0],
                             'bbl'))

        with codecs.open(self.app.options.master, 'r', encoding='utf-8') as f:
            root_text = f.read()
        tex = inline(root_text)
        tex = remove_comments(tex)
        tex = self._process_figures(tex, dirname)
        if os.path.exists(bbl_path):
            with codecs.open(bbl_path, 'r', encoding='utf-8') as f:
                bbl_text = f.read()
            tex = inline_bbl(tex, bbl_text)
        else:
            self.log.debug("Skipping .bbl installation")

        if self._build_style == "aastex":
            output_tex_path = os.path.join(dirname, "ms.tex")
        else:
            output_tex_path = os.path.join(
                dirname,
                os.path.basename(self.app.options.master))
        self._write_tex(tex, output_tex_path)

    def _process_figures(self, tex, dirname):
        """Discover figures and copy to root of build directory.

        Returns
        -------
        figs : dict
            A dictionary of with figure names (without extension or directory)
            as the key and the current full path as the value.
        """
        figs = discover_figures(tex, self._ext_priority)
        if self._build_style == "aastex":
            maxsize = None
        elif self._build_style == "arxiv":
            maxsize = self._max_size

        tex = install_figs(
            tex, figs, dirname,
            naming=self._build_style,
            format_priority=self._ext_priority,
            max_size=maxsize)
        return tex

    def _write_tex(self, tex, path):
        """Write the LaTeX to the output path."""
        with codecs.open(path, 'w', encoding='utf-8') as f:
            f.write(tex)


def discover_figures(tex, ext_priority):
    """Find all figures in the manuscript.

    Returns
    -------
    figs : dict
        A dictionary with figure names (without extension or directory)
        as the key and and values are dicts with keys: path, options and
        figure environment.
    """
    figs_pattern = re.compile(r"\\includegraphics(.*?){(.*?)}", re.UNICODE)
    matches = figs_pattern.findall(tex)
    figs = {}
    for i, match in enumerate(matches):
        opts, path = match
        basename = os.path.splitext(os.path.basename(path))[0]
        # Find all formats this file exists in
        exts = _find_exts(path, ext_priority)
        # Get file sizes for all variants here
        _dir = os.path.dirname(path)
        sizes = []
        for ext in exts:
            p = os.path.join(_dir, ".".join((basename, ext)))
            if os.path.exists(p):
                sizes.append(os.path.getsize(p) / 10. ** 6.)
            else:
                sizes.append(0.0)
        figs[basename] = {"path": path,
                          "exts": exts,
                          "size_mb": sizes,
                          "options": opts,
                          "env": r"\\includegraphics",
                          "num": i + 1}
    return figs





def install_figs(tex, figs, install_dir, naming=None,
                 format_priority=('pdf', 'eps', 'ps', 'png', 'jpg', 'tif'),
                 max_size=None):
    """Copy each figure to the build directory and update tex with new path.

    Parameters
    ----------
    tex : unicode
        The tex document as a unicode string.
    figs : dict
        A dictionary with figure names (without extension or directory)
        as the key and and values are dicts with keys: path, options and
        figure environment.
    naming : str
        Style for figure naming, (``'aastex'|'arxiv'|None``).
    format_priority : list
        List of figure file extensions, in order of preference to use in
        the final build.
    max_size : float
        Maximum size for a figure before converting it into a JPEG.
        If ``None``, no conversions are attempted.
    """
    for figname, fig in figs.items():
        if len(fig['exts']) == 0:
            continue
        # get the priority graphics file type
        full_path = None
        for ext in format_priority:
            if ext in fig['exts']:
                figsize = fig['size_mb'][fig['exts'].index(ext)]
                full_path = ".".join((os.path.splitext(fig['path'])[0], ext))
                break
        if full_path is None:
            continue

        # copy fig to the build directory
        if naming == "aastex":
            install_path = os.path.join(
                install_dir,
                f"f{fig['num']:d}.{ext}")
        elif naming == "arxiv":
            install_path = os.path.join(
                install_dir,
                f"figure{fig['num']:d}.{ext}")
        else:
            install_path = os.path.join(
                install_dir,
                os.path.basename(full_path))
        figs[figname]["installed_path"] = install_path
        try:
            shutil.copy(full_path, install_path)
        except FileNotFoundError:
            continue
        if max_size and figsize > max_size:
            rasterize_figure(install_path)
        # update tex by replacing old filename with new.
        # Note that fig['env'] currently has escaped slash for re; this is
        # removed here. Might want to think of a convention so it's less kludgy
        old_fig_cmd = r"{env}{opts}{{{path}}}".format(
            env=fig['env'].replace("\\\\", "\\"),
            opts=fig['options'],
            path=fig['path'])
        new_fig_cmd = r"{env}{opts}{{{path}}}".format(
            env=fig['env'].replace("\\\\", "\\"),
            opts=fig['options'],
            path=os.path.basename(os.path.splitext(install_path)[0]))
        tex = tex.replace(old_fig_cmd, new_fig_cmd)
    return tex


def rasterize_figure(original_path):
    """Make a JPEG version of a figure, deleting the original."""
    jpg_path = os.path.splitext(original_path)[0] + ".jpg"
    subprocess.call(
        "convert -density 300 -trim -quality 80 {path} {jpgpath}".format(
            path=original_path, jpgpath=jpg_path),
        shell=True)
    os.remove(original_path)



#!/usr/bin/env python
# encoding: utf-8
import logging
import os
import json

from cliff.command import Command
from .texutils import find_root_tex_document, RootNotFound

from preprint.config import Configurations


class Init(Command):
    """Initialze the project with preprint.json configurations."""

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(Init, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        write_configs()
        self.log.info("Wrote preprint.json")


def write_configs():
    """Write a default configurations file for the current project."""
    try:
        root_tex = find_root_tex_document(base_dir=".")
    except RootNotFound:
        root_tex = "article.tex"
    configs = Configurations()
    config_dict = configs.default_dict
    config_dict['master'] = root_tex
    if os.path.exists("preprint.json"):
        os.remove("preprint.json")
    with open("preprint.json", 'w') as f:
        f.write(json.dumps(config_dict,
                           sort_keys=True,
                           indent=4,
                           separators=(',', ': ')))



#!/usr/bin/env python
# encoding: utf-8
"""
Tests for regular expression detection of ``\inline{*}`` commands.
"""

from preprint.texutils import inline


def _mock_sub_line(match):
    return u"Replaced"


def test_standard_inline():
    """Test :func:`inline` to detecting ``\inline{*}`` commands."""
    data = u"Keep\n\input{test}"
    processed_data = inline(data, replacer=_mock_sub_line)
    assert processed_data == u"Keep\nReplaced"


def test_ifexists_inline():
    """Test :func:`inline` to detecting ``\InputIfFileExists{*}`` commands."""
    data = u"Keep\n\InputIfFileExists{test}{a}{b}"
    processed_data = inline(
        data, ifexists_replacer=_mock_sub_line, replacer=_mock_sub_line)
    assert processed_data == u"Keep\nReplaced"



#!/usr/bin/env python
# encoding: utf-8
import logging
import subprocess

from cliff.command import Command

from .vc import run_vc


class Make(Command):
    """Do a one-off compilation of the paper"""

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(Make, self).get_parser(prog_name)
        parser.add_argument(
            '--cmd',
            default=self.app.confs.config('cmd'),
            help="Command to run for compilation")
        return parser

    def take_action(self, parsed_args):
        run_vc()
        cmd = parsed_args.cmd.format(master=self.app.options.master)
        self.log.debug("Compiling with {0}".format(cmd))
        subprocess.call(cmd, shell=True)



#!/usr/bin/env python
# encoding: utf-8
"""
LaTeX utilities.
"""

import os
import re
import logging
from TexSoup import TexSoup

from .gitio import read_git_blob


class RootNotFound(Exception):
    pass


def remove_comments(tex):
    """Remove comments from LaTeX."""
    return re.sub(r'(?<!\\)%.*', '', tex)


def inline(tex, base_dir='.', replacer=None, ifexists_replacer=None):
    """Inline LaTeX files."""

    def sub_if_exists(match):
        file_path = os.path.join(base_dir, match.group(1) + '.tex')
        if not os.path.exists(file_path):
            file_path = os.path.join('.', match.group(1) + '.tex')

        if os.path.exists(file_path):
            with open(file_path, 'r') as f:
                content = f.read()
            return inline(content,
                          base_dir=os.path.dirname(file_path),
                          replacer=replacer,
                          ifexists_replacer=ifexists_replacer)
        return ''

    if replacer is None:
        replacer = sub_if_exists

    if ifexists_replacer is None:
        ifexists_replacer = sub_if_exists

    tex = re.sub(r'\\input{(.*?)}', replacer, tex)
    tex = re.sub(r'\\InputIfFileExists{(.*?)}{.*?}{.*?}', ifexists_replacer, tex)
    return tex


def inline_bbl(tex, bbl_text):
    """Inline a .bbl file."""
    soup = TexSoup(tex)
    
    # Find the \bibliography command and replace it with the bbl content
    bib_node = soup.bibliography
    if bib_node:
        bib_node.replace_with(bbl_text)
    else:
        # If no \bibliography command is found, try to find a \begin{document}
        # and insert the bbl_text before it, as a fallback.
        # This is a less ideal, but might work for some cases.
        begin_document = soup.find('document')
        if begin_document:
            begin_document.insert_before(bbl_text)

    return str(soup)


def inline_blob(commit_ref, tex, base_dir='.', repo_dir='.'):
    """Inline LaTeX files from a git blob."""
    soup = TexSoup(tex)
    for node in soup.find_all(('input', 'InputIfFileExists')):
        if not node.args:
            continue

        file_path = os.path.join(base_dir, node.args[0] + '.tex')
        try:
            content = read_git_blob(commit_ref, file_path, repo_dir=repo_dir)
            # Recursively inline content
            content = inline_blob(commit_ref, content,
                                  base_dir=os.path.dirname(file_path),
                                  repo_dir=repo_dir)
            node.replace_with(content)
        except Exception:
            # If the file does not exist in the blob, replace with empty string
            node.replace_with('')
    return str(soup)


def find_root_tex_document(base_dir='.'):
    """Find the root .tex document in a directory."""
    for root, dirs, files in os.walk(base_dir):
        for file in files:
            if file.endswith('.tex') and 'main' in file.lower(): # Simple heuristic
                return os.path.join(root, file)
    raise RootNotFound("No root .tex document found.")


def _find_exts(fig_path, ext_priority):
    """Return a tuple of all formats for which a figure exists."""
    basepath = os.path.splitext(fig_path)[0]
    has_exts = []
    for ext in ext_priority:
        p = ".".join((basepath, ext))
        if os.path.exists(p):
            has_exts.append(ext)
    return tuple(has_exts)


import logging
import sys

from cliff.app import App
from cliff.commandmanager import CommandManager

from .config import Configurations


VERSION = "0.3.1"


class PreprintApp(App):

    log = logging.getLogger(__name__)
    confs = Configurations()

    def __init__(self):
        super(PreprintApp, self).__init__(
            description='Tools for writing latex papers',
            version=VERSION,
            command_manager=CommandManager('preprint.commands'))
        self.log.debug('Commands loaded: %s', sorted(self.command_manager))

    def initialize_app(self, argv):
        self.log.debug('initialize_app')

    def build_option_parser(self, *args):
        parser = super(PreprintApp, self).build_option_parser(*args)
        parser.add_argument(
            '--master',
            default=self.confs.config('master'),
            help='Name of master tex file')
        return parser

    def prepare_to_run_command(self, cmd):
        self.log.debug('prepare_to_run_command %s', cmd.__class__.__name__)

    def clean_up(self, cmd, result, err):
        self.log.debug('clean_up %s', cmd.__class__.__name__)
        if err:
            self.log.debug('got an error: %s', err)


def main(argv=sys.argv[1:]):
    myapp = PreprintApp()
    return myapp.run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



import logging
import os
import subprocess
import time

from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

from cliff.command import Command

from preprint.latexdiff import git_diff_pipeline
from .vc import run_vc


class Watch(Command):
    """Watch for changes and compile paper"""

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(Watch, self).get_parser(prog_name)
        parser.add_argument(
            '--exts',
            nargs='*',
            default=self.app.confs.config('exts'),
            help="File extensions to look for")
        parser.add_argument(
            '--cmd',
            default=self.app.confs.config('cmd'),
            help="Command to run on changes")
        parser.add_argument(
            '--diff',
            nargs='?',
            const='HEAD',
            default=None,
            help="Typeset diff against git commit")
        return parser

    def take_action(self, parsed_args):
        ignore = (os.path.splitext(self.app.options.master)[0] + ".pdf",
                  'build', '_current.tex', '_prev.tex')
        if parsed_args.diff is None:
            handler = RegularChangeHandler(
                parsed_args.cmd, parsed_args.exts, ignore)
        else:
            handler = DiffChangeHandler(
                self.app.options.master, parsed_args.diff, parsed_args.exts,
                ignore)
        self._watch(handler)

    def _watch(self, handler):
        observer = Observer()
        observer.schedule(handler, '.', recursive=True)
        observer.start()
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()
        observer.join()


class BaseChangeHandler(FileSystemEventHandler):
    """React to modified files."""
    def __init__(self, exts, ignores):
        super(BaseChangeHandler, self).__init__()
        self._exts = exts
        self._ignores = ignores

    def on_any_event(self, event):
        """If a file or folder is changed."""
        if event.is_directory:
            return
        else:
            event_ext = os.path.splitext(event.src_path)[-1]\
                .lower().lstrip('.')
            if event_ext in self._exts:
                for ig in self._ignores:
                    if ig in event.src_path:
                        return
                # passed all tests
                self.run_compile()
        return


class RegularChangeHandler(BaseChangeHandler):
    """Class for reacting to modified files and doing a regular compile."""
    def __init__(self, command, exts, ignores):
        super(RegularChangeHandler, self).__init__(exts, ignores)
        self._cmd = command

    def run_compile(self):
        """Run a compilation."""
        run_vc()
        subprocess.call(self._cmd, shell=True)


class DiffChangeHandler(BaseChangeHandler):
    """React to modified files while building latexdiffs."""
    def __init__(self, master_path, prev_commit, exts, ignores):
        super(DiffChangeHandler, self).__init__(exts, ignores)
        self._master = master_path
        self._prev_commit = prev_commit
        self._output_name = "{0}_diff".format(
            os.path.splitext(self._master)[0])
        # Hack the ignore list to include the output path
        self._ignores = list(ignores)
        self._ignores.append(self._output_name)

    def run_compile(self):
        """Run a latexdiff+compile."""
        git_diff_pipeline(
            self._output_name, self._master,
            self._prev_commit)



#!/usr/bin/env python

import os
import re
from setuptools import setup, find_packages, Command

# from setuptools.core import setup, Command
# you can also import from setuptools


def rel_path(path):
    return os.path.join(os.path.dirname(__file__), path)


def get_version():
    with open(rel_path(os.path.join("preprint", "main.py"))) as f:
        for line in f:
            if line.startswith("VERSION"):
                version = re.findall(r'\"(.+?)\"', line)[0]
                return version
    return "0.0.0.dev"


try:
    long_description = open(rel_path('README.md'), 'rt').read()
except IOError:
    long_description = ''


setup(
    name='preprint3',
    version=get_version(),

    description='Tools for writing latex papers',
    long_description=long_description,
    long_description_content_type='text/markdown',

    author='Jonathan Sick',
    author_email='jonathansick@mac.com',

    url='https://github.com/jonathansick/preprint',
    download_url='',

    classifiers=['Development Status :: 3 - Alpha',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.10',
                 'Environment :: Console',
                 ],

    platforms=['Any'],

    scripts=[],

    provides=[],
    install_requires=['cliff',
                      'watchdog',
                      'GitPython',
                      'pytest',
                      'TexSoup'],

    namespace_packages=[],
    packages=find_packages(),
    include_package_data=True,



    entry_points={
        'console_scripts': [
            'preprint = preprint.main:main'
        ],
        'preprint.commands': [
            'init = preprint.init:Init',
            'make = preprint.make:Make',
            'watch = preprint.watch:Watch',
            'diff = preprint.latexdiff:Diff',
            'pack = preprint.pack:Package',
        ],
    },

    zip_safe=False,
)



#!/usr/bin/env python
# encoding: utf-8
"""
Manager for configuration defaults.
"""

import os
import json


class Configurations(object):
    """Configurations determines and provides default settings that can
    be overriden by the user on the command line.

    Configurations are set at two levels:

    1. There are built-in defaults that ensure all configurations are
        always set.
    2. These can be overriden by settings in a json file (an example is below).

    Each command uses these configurations to set the default state of
    each command line option. Thus each command ultimatly gets the
    final configuration state from the argparser.

    An example json file, named "preprint.json":
    
    ::
        {
            "master": "skysub.tex",
            "exts": ["tex", "eps", "pdf"],
            "cmd": "latexmk -f -pdf -bibtex-cond {master}"
        }


    *Notes on the ``cmd`` option:* this open accepts a ``master`` template
    variable that will be replaced with the value of the ``master``
    configuration variable. This can be used to tell the appropriate latex
    build command what the master tex file is (see example above).
    """

    _DEFAULTS = {
        "master": "paper.tex",
        "exts": ["tex", "pdf", "eps"],
        "cmd": "latexmk -f -pdf -bibtex-cond {master}"}

    def __init__(self):
        super(Configurations, self).__init__()
        self._confs = dict(self._DEFAULTS)
        # Read configurations
        if os.path.exists("preprint.json"):
            with open("preprint.json", 'r') as f:
                self._confs.update(json.load(f))
        self._sanitize_path('master')

    def default(self, name):
        """Get the default value for the named config, given the section."""
        return self._DEFAULTS[name]

    def config(self, name):
        """Get the configuration."""
        if name == "cmd":
            return self._confs['cmd'].format(master=self._confs['master'])
        else:
            return self._confs[name]

    @property
    def default_dict(self):
        return dict(self._DEFAULTS)

    def _sanitize_path(self, key):
        """Sanitize the path of a configuration given `key`."""
        p = self._confs[key]
        p = os.path.expandvars(os.path.expanduser(p))
        if os.path.dirname(p) == ".":
            p = os.path.basename(p)
        self._confs[key] = p


if __name__ == '__main__':
    conf = Configurations()
    print(conf.default("master"))
    print(conf.default("exts"))
    print(conf.default("cmd"))
    print(conf.config("exts"))
    print(type(conf.config("exts")))
    print(conf.config("cmd"))



#!/usr/bin/env python
# encoding: utf-8
"""
Runs the vc tool, if available, to update the version control string in your
latex document.

See http://www.ctan.org/pkg/vc
"""

import os
import subprocess
import logging


log = logging.getLogger(__name__)


def vc_exists():
    """Return `True` if the project uses vc."""
    if os.path.exists('vc') and os.path.exists('vc-git.awk'):
        log.debug("Found a vc installation")
        return True
    else:
        log.debug("Did not find a vc installation")
        return False


def run_vc():
    """Run the vc tool."""
    if vc_exists():
        log.debug("Running vc")
        subprocess.call("./vc", shell=True)
